@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.h 1.1 1997/01/11 13:09:34 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.h 1.3 1997/01/25 12:38:21 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 0.9.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
   Error logging header file.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -22,7 +22,7 @@
 
 /* In errors.c */
 
-void InitErrorHandler(char *argv);
+void InitErrorHandler(char *name);
 void PrintMessage(int errlev,char* fmt, ...);
 
 #endif /* ERRORS_H */
