@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/parse.c 1.1 1997/01/11 13:21:42 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/parse.c 1.6 1997/01/24 17:31:26 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 0.9.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
   Functions to parse the HTTP requests.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -23,45 +23,105 @@
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Read from the specified file descriptor until a URL is seen.
+  Parse the request to the server.
 
-  char *GetURL Returns the line that was read.
+  char *ParseRequest Returns the request.
 
   FILE *req The file pointer to read the request from.
 
   char **url Return the url or NULL if it failed.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *GetURL(FILE *req,char **url)
+char *ParseRequest(FILE *req,char **url)
 {
- int start=0,stop=0;
- char *buffer=NULL;
+ char *buffer=NULL,*buf=NULL;
+ int post=0,n=0,length=0;
 
  *url=NULL;
 
- buffer=fgets_realloc(buffer,req);
+ buf=fgets_realloc(buf,req);
 
- if(!buffer)
+ if(!buf)
     return(NULL);
 
- /* Make a copy of the URL from the buffer */
+ *url=(char*)malloc(strlen(buf));
 
- for(start=0;buffer[start]!='\n';start++)
-    if(buffer[start]==' ' || buffer[start]=='\t')
-       break;
+ if(sscanf(buf,"%*s %s",*url)!=1)
+   {free(*url);*url=NULL;return(NULL);}
 
- while(buffer[++start]!='\n' && (buffer[start]==' ' || buffer[start]=='\t'));
+ if(!strncasecmp("POST",buf,4))
+    post=1;
 
- if(buffer[start]=='\n')
-    return(buffer);
+ buffer=(char*)malloc(strlen(buf)+1);
+ buffer[0]=0;
 
- for(stop=start;buffer[stop]!='\n';stop++)
-    if(buffer[stop]==' ' || buffer[stop]=='\t' || buffer[stop]=='\r')
-       break;
+ do
+   {
+    char *pling;
 
- *url=(char*)malloc(stop-start+1);
- strncpy(*url,&buffer[start],stop-start);
- (*url)[stop-start]=0;
+    if(!strncasecmp("Referer:",buf,8) && (pling=strstr(buf,"?!")))
+       strcpy(pling,"\r\n");
+
+    if(n)
+       buffer=(char*)realloc(buffer,n+strlen(buf)+1);
+    strcpy(&buffer[n],buf);
+    n+=strlen(buf);
+
+    if(post && !strncasecmp("Content-length:",buf,15))
+       length=atoi(&buf[15]);
+   }
+ while(buf && buf[0]!='\r' && buf[0]!='\n' && (buf=fgets_realloc(buf,req)));
+
+ if(buf)
+    free(buf);
+
+ if(post)
+   {
+    int m;
+
+    if(!length)
+      {free(*url);*url=NULL;return(NULL);}
+
+    buffer=(char*)realloc(buffer,n+length+1);
+    m=fread(&buffer[n],1,length,req);
+
+    if(m!=length)
+      {free(*url);*url=NULL;return(NULL);}
+
+    buffer[n+length]=0;
+
+    *url=(char*)realloc(*url,strlen(*url)+20);
+    sprintf(*url,"%s?!%08lx.%04lx",*url,MakeHashFromArgs(&buffer[n]),(unsigned long)length);
+   }
+
+ return(buffer);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Parse the reply from the server.
+
+  char *ParseReply Return the chunk of data that was read.
+
+  int rep The file descriptor to read the reply from.
+
+  int *status Return the numeric status code.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *ParseReply(int rep,int *status)
+{
+ char *buffer=(char*)malloc(65);
+ int n;
+
+ n=ReadOrTimeout(rep,buffer,64);
+
+ if(n<=0)
+    return(NULL);
+
+ buffer[n]=0;
+
+ if(sscanf(buffer,"%*s %d",status)!=1)
+    *status=0;
 
  return(buffer);
 }
@@ -82,15 +142,13 @@ char *GetURL(FILE *req,char **url)
 void SplitURL(char *url,char **host,char **path,char **args)
 {
  int i=0;
- char *copyurl=malloc(strlen(url)+1);
+ char *copyurl=malloc(strlen(url)+2);
 
  strcpy(copyurl,url);
  *host=NULL;
  *path=NULL;
  *args=NULL;
 
- /* Split up the copy of the URL */
-
  if(!strncasecmp(copyurl,"http:",5))
    {
     i=5;
@@ -117,7 +175,6 @@ void SplitURL(char *url,char **host,char **path,char **args)
     if(copyurl[i])
       {
        copyurl[i++]=0;
-
        *args=&copyurl[i];
       }
    }
@@ -137,36 +194,35 @@ void SplitURL(char *url,char **host,char **path,char **args)
   char *args The arguments to the URL we are going to get.
 
   char **buffer The buffer containing the request.
-
-  int *n The number of bytes in the buffer.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void MakeRequestNonProxy(char *url,char *path,char *args,char **buffer,int *n)
+void MakeRequestNonProxy(char *url,char *path,char *args,char **buffer)
 {
- char *start=strstr(*buffer,url);
- char *stop=start+strlen(url);
+ char *start,*stop;
 
- strcpy(start,"/");
- start++;
-
- if(*path)
+ if(args && *args=='!')
    {
-    strcpy(start,path);
-    start+=strlen(path);
+    char *pling=strstr(url,"?!");
+    *pling=0;
+    start=strstr(*buffer,url);
+    stop=start+strlen(url);
+    *pling='?';
+   }
+ else
+   {
+    start=strstr(*buffer,url);
+    stop=start+strlen(url);
    }
 
- if(args)
-   {
-    strcpy(start,"?");
-    start++;
-    strcpy(start,args);
-    start+=strlen(args);
-   }
+ if(args && *args!='!')
+    sprintf(start,"/%s?%s",path,args);
+ else
+    sprintf(start,"/%s",path);
+
+ start+=strlen(start);
 
  while(*stop)
     *start++=*stop++;
 
  *start=0;
-
- *n=start-*buffer;
 }
