@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 1.1 1997/01/11 13:09:34 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 1.4 1997/01/22 20:18:31 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 0.9.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
   A user level program to interact with the server.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -15,6 +15,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <unistd.h>
 
@@ -26,27 +27,17 @@
 static void usage(int verbose);
 
 
-/*+ The action to perform.
-  1=online, 2=offline, 3=fetch, 4=purge. +*/
-static int action=0;
-
-/*+ The port number that the demon listens on. +*/
-static int port=DEF_WWWOFFLE_PORT;
-
-/*+ The age at which files are to be purged. +*/
-static int purge_age=DEF_AGE;
-
-/*+ The number of hosts in the list to be purged. +*/
-static int purge_nhosts=0;
-
-/*+ The list of hosts that are to be purged. +*/
-static char **purge_hosts=NULL;
-
-/*+ The number of web pages to get. +*/
-static int www_npages=0;
-
-/*+ The web pages to get. +*/
-static char **www_pages=NULL;
+/*+ The action to perform. +*/
+typedef enum _Action
+{
+ None,                          /*+ Undecided. +*/
+ Online,                        /*+ Tell the server that we are online. +*/
+ Offline,                       /*+ Tell the server that we are offline. +*/
+ Fetch,                         /*+ Tell the server to fetch the reqeusted pages. +*/
+ Purge,                         /*+ Tell the server to purge pages. +*/
+ Get                            /*+ Tell the server to get pages. +*/
+}
+Action;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -55,10 +46,18 @@ static char **www_pages=NULL;
 
 int main(int argc, char** argv)
 {
+ int www_npages=0;
+ char **www_pages=NULL;
+
+ int purge_age=DEF_AGE;
+ int purge_nhosts=0;
+ char **purge_hosts=NULL;
+
+ Action action=None;
+
+ int port=-1;
+
  int i;
- int socket;
- char *buffer=NULL;
- FILE *server_r,*server_w;
 
  /* Parse the command line options */
 
@@ -93,19 +92,19 @@ int main(int argc, char** argv)
    {
     if(argc!=2)
       {fprintf(stderr,"wwwoffle: The '-online' option takes no arguments.\n"); exit(1);}
-    action=1;
+    action=Online;
    }
  else if(!strcmp(argv[i],"-off") || !strcmp(argv[i],"-offline"))
    {
     if(argc!=2)
       {fprintf(stderr,"wwwoffle: The '-offline' option takes no arguments.\n"); exit(1);}
-    action=2;
+    action=Offline;
    }
  else if(!strcmp(argv[i],"-fetch"))
    {
     if(argc!=2)
       {fprintf(stderr,"wwwoffle: The '-fetch' option takes no arguments.\n"); exit(1);}
-    action=3;
+    action=Fetch;
    }
  else if(!strcmp(argv[i],"-purge"))
    {
@@ -117,7 +116,12 @@ int main(int argc, char** argv)
        sprintf(ages,"%d",age);
 
        if(!strcmp(argv[i],ages))
+         {
           purge_age=age;
+
+          if(purge_age<0)
+            {fprintf(stderr,"wwwoffle: The age for purging (%d) must be positive.\n",age); exit(1);}
+         }
        else
          {
           if(!purge_hosts)
@@ -132,7 +136,7 @@ int main(int argc, char** argv)
     if(purge_age==0 && !purge_hosts)
       {fprintf(stderr,"wwwoffle: Cannot purge all pages, age=0 requires hostnames.\n"); exit(1);}
 
-    action=4;
+    action=Purge;
    }
  else if(argv[i][0]=='-')
    {fprintf(stderr,"wwwoffle: Unknown option '%s'.\n",argv[1]); exit(1);}
@@ -148,55 +152,103 @@ int main(int argc, char** argv)
        www_pages[www_npages++]=argv[i];
       }
 
-    action=5;
+    action=Get;
    }
 
  /* Initialise things. */
 
- InitErrorHandler(argv[0]);
+ InitErrorHandler("wwwoffle");
 
- socket=OpenClientSocket("localhost",port);
+ /* The connections to the WWWOFFLE server. */
 
- if(socket==-1)
-    PrintMessage(Fatal,"Cannot open connection to server.");
-
- server_r=fdopen(socket,"r");
- server_w=fdopen(socket,"w");
-
- /* Send the message. */
-
- if(action==1)
-    fprintf(server_w,"WWWOFFLE ONLINE\r\n");
- else if(action==2)
-    fprintf(server_w,"WWWOFFLE OFFLINE\r\n");
- else if(action==3)
-    fprintf(server_w,"WWWOFFLE FETCH\r\n");
- else if(action==4)
+ if(action!=Get)
    {
-    fprintf(server_w,"WWWOFFLE PURGE AGE %d\r\n",purge_age);
+    int socket;
+    FILE *server_r,*server_w;
+    char *buffer=NULL;
 
-    if(purge_hosts)
-       for(i=0;i<purge_nhosts;i++)
-          fprintf(server_w,"WWWOFFLE PURGE HOST %s\r\n",purge_hosts[i]);
+    if(port==-1)
+       port=DEF_WWWOFFLE_PORT;
+
+    socket=OpenClientSocket("localhost",port);
+
+    if(socket==-1)
+       PrintMessage(Fatal,"Cannot open connection to server.");
+
+    server_r=fdopen(socket,"r");
+    server_w=fdopen(socket,"w");
+
+    /* Send the message. */
+
+    if(action==Online)
+       fprintf(server_w,"WWWOFFLE ONLINE\r\n");
+    else if(action==Offline)
+       fprintf(server_w,"WWWOFFLE OFFLINE\r\n");
+    else if(action==Fetch)
+       fprintf(server_w,"WWWOFFLE FETCH\r\n");
+    else if(action==Purge)
+      {
+       fprintf(server_w,"WWWOFFLE PURGE AGE %d\r\n",purge_age);
+
+       if(purge_hosts)
+          for(i=0;i<purge_nhosts;i++)
+             fprintf(server_w,"WWWOFFLE PURGE HOST %s\r\n",purge_hosts[i]);
+
+       free(purge_hosts);
+      }
+
+    fprintf(server_w,"\r\n");
+    fflush(server_w);
+
+    while((buffer=fgets_realloc(buffer,server_r)))
+       fputs(buffer,stdout);
+
+    fclose(server_w);
+    fclose(server_r);
    }
- else /* action==5 */
-   {
-    for(i=0;i<www_npages;i++)
-       fprintf(server_w,"WWWOFFLE GET %s\r\n",www_pages[i]);
 
+ /* The connections to the http proxy. */
+
+ else /* action==Get */
+   {
+    int socket;
+    char *buffer=(char*)malloc(257);
+
+    if(port==-1)
+       port=DEF_HTTP_PORT;
+
+    for(i=0;i<www_npages;i++)
+      {
+       char *request=(char*)malloc(strlen(www_pages[i])+48);
+
+       socket=OpenClientSocket("localhost",port);
+
+       if(socket==-1)
+          PrintMessage(Fatal,"Cannot open connection to server.");
+
+       if(!strncmp(www_pages[i],"http:",5))
+         {
+          www_pages[i]+=5;
+          if(*www_pages[i]=='/') www_pages[i]++;
+          if(*www_pages[i]=='/') www_pages[i]++;
+         }
+
+       printf("Getting http://%s\n",www_pages[i]);
+
+       sprintf(request,"GET /outgoing/%s HTTP/1.0\r\nAccept: */*\r\n\r\n",www_pages[i]);
+       write(socket,request,strlen(request));
+
+       while(read(socket,buffer,256)>0);
+
+       close(socket);
+       free(request);
+      }
+
+    free(buffer);
     free(www_pages);
    }
 
- fprintf(server_w,"\r\n");
- fflush(server_w);
-
- while((buffer=fgets_realloc(buffer,server_r)))
-    fputs(buffer,stdout);
-
- /* Close down and exit. */
-
- fclose(server_w);
- fclose(server_r);
+ /* exit. */
 
  return(0);
 }
@@ -211,7 +263,7 @@ int main(int argc, char** argv)
 static void usage(int verbose)
 {
  fputs("\n"
-       "WWWOFFLE - World Wide Web Offline Explorer - Version 0.9\n"
+       "WWWOFFLE - World Wide Web Offline Explorer - Version 1.0\n"
        "\n",stderr);
 
  if(verbose)
@@ -246,8 +298,8 @@ static void usage(int verbose)
             "wwwoffle <url>         : Fetch the specified URL (or record for fetching).\n"
             "\n"
             "     -port <port.      : The port number to talk to the demon on.\n"
-            "                         (Default %d).\n"
-            "\n",port);
+            "                         (Defaults %d for server, %d for http proxy).\n"
+            "\n",DEF_WWWOFFLE_PORT,DEF_HTTP_PORT);
 
  exit(0);
 }
