@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.1 1997/01/11 13:09:34 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.4 1997/01/22 20:21:58 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 0.9.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
   A header file for all of the programs wwwoffle, wwwoffled, wwwoffles.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -36,8 +36,8 @@
 /*+ The character to use to replace '/' in the spool directories. +*/
 #define PATH_SEP '%'
 
-/*+ The time in seconds to wait while reading from the server. +*/
-#define SERVER_TIMEOUT 120
+/*+ The time in seconds to wait while reading from a socket. +*/
+#define SOCKET_TIMEOUT 120
 
 /* In parse.c */
 
@@ -51,27 +51,30 @@ void PurgeCache(int age,int nhosts,char **hosts,int client);
 /* In spool.c */
 
 int OpenOutgoingSpoolFile(int rw);
-int OpenWebpageSpoolFile(int rw,char *url);
+int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args);
+unsigned long MakeHashFromArgs(char *args);
 
 /* In parse.c */
 
-char *GetURL(FILE *req,char **url);
+char *ParseRequest(FILE *req,char **url);
+char *ParseReply(int rep,int *status);
 void SplitURL(char *url,char **host,char **path,char **args);
-void MakeRequestNonProxy(char *url,char *path,char *args,char **buffer,int *n);
+void MakeRequestNonProxy(char *url,char *path,char *args,char **buffer);
 
 /* In messages.c */
 
 void ServerErrorMessage(int fd,char *string);
 void ServerFetchFailed(int fd,char *url,char *host,int port,int connect_or_timeout);
 void WillGet(int fd,char *url,int already);
+void Refresh(int fd,char *url);
 
 /* In index.c */
 
-void GenerateIndex(int fd,char *url);
+void GenerateIndex(int fd,char *path,char *args);
 
 /* In io.c */
 
 char *fgets_realloc(char *buffer,FILE *file);
-int ReadOrTimeout(int fd,char *buffer,int n,int timeout);
+int ReadOrTimeout(int fd,char *buffer,int n);
 
 #endif /* WWWOFFLE_H */
